/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#define	FE_HD_AUTOROTATE	FALSE	// convert for z-up to y-up

#include "houdini.pmh"

namespace fe
{

void HoudiniDraw::drawPoints(const SpatialVector *vertex,
	const SpatialVector *normal,U32 vertices,
	BWORD multicolor,const Color *color)
{
	if(!gdp())
	{
		feLog("HoudiniDraw::drawPoints gdp is NULL\n");
		return;
	}

	for(U32 index=0;index<vertices;index++)
	{
		const SpatialVector& vert=vertex[index];

#if FE_HD_AUTOROTATE
		UT_Vector3 utPoint3(vert[1],vert[2],vert[0]);
#else
		UT_Vector3 utPoint3(vert[0],vert[1],vert[2]);
#endif

		GEO_Point* pGeoPoint=gdp()->appendPoint();

		pGeoPoint->setPos(utPoint3);
	}
}

void HoudiniDraw::drawLines(const SpatialVector *vertex,
	const SpatialVector *normal,U32 vertices,
	StripMode strip,BWORD multicolor,
	const Color *color)
{
	if(!gdp())
	{
		feLog("HoudiniDraw::drawLines gdp is NULL\n");
		return;
	}

	sp<DrawMode> spDrawMode=drawMode();
	String group;
	if(spDrawMode.isValid())
	{
		group=spDrawMode->group();
	}

	const U32 subCount=(strip==DrawI::e_strip)? vertices: 2;
	for(U32 index=0;index<vertices;index+=subCount)
	{
		GB_PointGroup* pPointGroup=m_pGdp->findPointGroup(group.c_str());
		if(!pPointGroup)
		{
			pPointGroup=m_pGdp->newPointGroup(group.c_str());
		}
		FEASSERT(pPointGroup);

		int npts=subCount;
		int open=GU_POLY_OPEN;
		int appendPts=1;
		GU_PrimPoly* pGuPrimPoly=
				GU_PrimPoly::build(gdp(),npts,open,appendPts);

		GEO_AttributeOwner owner=GEO_POINT_DICT;
		GEO_AttributeHandle handle;
		if(color)
		{
			handle=m_pGdp->getAttribute(owner,"Cd");
		}

		for(U32 subIndex=0;subIndex<subCount;subIndex++)
		{
			const SpatialVector& vert=vertex[index+subIndex];

#if FE_HD_AUTOROTATE
			UT_Vector3 utPoint3(vert[1],vert[2],vert[0]);
#else
			UT_Vector3 utPoint3(vert[0],vert[1],vert[2]);
#endif

			GEO_Point* pGeoPoint=gdp()->appendPoint();

			pGeoPoint->setPos(utPoint3);

			pPointGroup->add(pGeoPoint);

			if(color)
			{
				const SpatialVector& color0=color[index+subIndex];

				handle.setElement(pGeoPoint);
				UT_Vector4 utVector4(color0.temp());
				handle.setV4(utVector4);
			}

			pGuPrimPoly->setVertex(subIndex, pGeoPoint);
		}
	}
}

} // namespace
