/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __surface_SurfaceBase_h__
#define __surface_SurfaceBase_h__

namespace fe
{

/**************************************************************************//**
	@brief Trivial Point Origin Surface

	@ingroup surface
*//***************************************************************************/
class FE_DL_EXPORT SurfaceBase: virtual public RecordableI,
		virtual public SurfaceI, virtual public DrawableI
{
	public:
							SurfaceBase(void):
								m_cached(FALSE)								{}
virtual						~SurfaceBase(void)								{}

							//* As Protectable
virtual	void				protect(void)
							{
//								feLog("SurfaceBase::protect checkCache\n");
								checkCache();
//								feLog("SurfaceBase::protect"
//										" Protectable::protect\n");
								Protectable::protect();
//								feLog("SurfaceBase::protect done\n");
							}

							//* As SurfaceI
virtual SpatialVector		center(void) const	{ return SpatialVector(0,0,0); }
virtual Real				radius(void) const	{ return Real(0); }
virtual Color				diffuse(void) const
							{	return Color(1,1,1,1); }

virtual SpatialTransform	sample(const SpatialTransform& a_transform,
									Vector2 a_uv) const
							{
								return a_transform;
							}
virtual SpatialTransform	sample(Vector2 a_uv) const
							{
								return SpatialTransform::identity();
							}

virtual SpatialVector		samplePoint(const SpatialTransform& a_transform,
									Vector2 a_uv) const
							{	return transformVector(a_transform,
										sample(a_uv).translation()); }
virtual SpatialVector		samplePoint(Vector2 a_uv) const
							{	return sample(a_uv).translation(); }

virtual SpatialVector		sampleNormal(const SpatialTransform& a_transform,
									Vector2 a_uv) const
							{	return rotateVector(a_transform,
										sample(a_uv).up()); }
virtual SpatialVector		sampleNormal(Vector2 a_uv) const
							{	return sample(a_uv).up(); }

virtual	sp<ImpactI>			nearestPoint(const SpatialTransform& a_transform,
									const SpatialVector& a_origin)
							{
								checkCache();
								return const_cast<const SurfaceBase*>(this)->
										nearestPoint(a_transform,a_origin);
							}
virtual	sp<ImpactI>			nearestPoint(const SpatialVector& a_origin)
							{
								checkCache();
								return const_cast<const SurfaceBase*>(this)->
										nearestPoint(a_origin);
							}
virtual	sp<ImpactI>			nearestPoint(const SpatialTransform& a_transform,
									const SpatialVector& a_origin) const
							{	return sp<ImpactI>(NULL); }
virtual	sp<ImpactI>			nearestPoint(const SpatialVector& a_origin) const
							{	return sp<ImpactI>(NULL); }

virtual	sp<ImpactI>			rayImpact(const SpatialTransform& a_transform,
									const SpatialVector& a_origin,
									const SpatialVector& a_direction)
							{
								checkCache();
								return const_cast<const SurfaceBase*>(this)->
										rayImpact(a_transform,
										a_origin,a_direction);
							}
virtual	sp<ImpactI>			rayImpact(const SpatialVector& a_origin,
									const SpatialVector& a_direction)
							{
								checkCache();
								return const_cast<const SurfaceBase*>(this)->
										rayImpact(a_origin,a_direction);
							}
virtual	sp<ImpactI>			rayImpact(const SpatialTransform& a_transform,
									const SpatialVector& a_origin,
									const SpatialVector& a_direction) const
							{	return sp<ImpactI>(NULL); }
virtual	sp<ImpactI>			rayImpact(const SpatialVector& a_origin,
									const SpatialVector& a_direction) const
							{	return sp<ImpactI>(NULL); }

							//* As DrawableI
virtual	void				draw(sp<DrawI> a_spDrawI,const fe::Color* color)
							{
								checkCache();
								const_cast<const SurfaceBase*>(this)->draw(
										a_spDrawI,color);
							}
virtual	void				draw(sp<DrawI> a_spDrawI,
									const fe::Color* color) const
							{
								SpatialTransform identity;
								setIdentity(identity);
								draw(identity,a_spDrawI,color);
							}
virtual	void				draw(const SpatialTransform& transform,
									sp<DrawI> a_spDrawI,const fe::Color* color)
							{
								checkCache();
								const_cast<const SurfaceBase*>(this)->
										draw(transform,a_spDrawI,color);
							}
virtual	void				draw(const SpatialTransform&,
									sp<DrawI>,const fe::Color*) const		{}

							//* as RecordableI
virtual	void				bind(Record& rRecord)							{}

	protected:
virtual	void				cache(void)										{}
virtual	void				cacheResolve(void)								{}
virtual	void				checkCache(void)
							{
								if(!m_cached)
								{
									cache();
									cacheResolve();
									m_cached=TRUE;
								}
							}

		BWORD				m_cached;
};

} // namespace

#endif /* __surface_SurfaceBase_h__ */

