/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __grass_Blade__
#define __grass_Blade__

namespace fe
{

/**************************************************************************//**
	@brief minimal semi-implicit curve sim

	@ingroup grass
*//***************************************************************************/
class FE_DL_EXPORT Blade
{
	public:

						Blade(void);
virtual					~Blade(void)										{}

		void			setTime(Real a_t)		{ m_t=a_t; }
		Real			time(void) const		{ return m_t; }

		void			setTimeStep(Real a_h)	{ m_h=a_h; }
		Real			timeStep(void) const	{ return m_h; }

		void			setTension(Real a_k)	{ m_k=a_k; }
		Real			tension(void) const		{ return m_k; }

		void			setRestoration(Real a_restoration)
						{	m_restoration=a_restoration; }
		Real			restoration(void) const
						{	return m_restoration; }

		void			setDrag(Real a_c)		{ m_c=a_c; }
		Real			drag(void) const		{ return m_c; }

		void			setRest(Real a_r)		{ m_r=a_r; }
		Real			rest(void) const		{ return m_r; }

		void			setMass(Real a_m)		{ m_m=a_m; }
		Real			mass(void) const		{ return m_m; }

		void			setWind(Real a_wind)
								{ m_wind=a_wind; }
		Real			wind(void) const
								{ return m_wind; }

		void			setTurbulence(Real a_turbulence)
								{ m_turbulence=a_turbulence; }
		Real			turbulence(void) const
								{ return m_turbulence; }

		void			setCenter(SpatialVector a_center)
								{ m_center=a_center; }
		SpatialVector	center(void) const
								{ return m_center; }

		void			setRadius(Real a_radius)
								{ m_radius=a_radius; }
		Real			radius(void) const
								{ return m_radius; }

		void			populate(void);
		void			step(U32 index,
								const std::vector<SpatialVector>& tangent,
								std::vector<SpatialVector>& location,
								std::vector<Real>& velocity) const;

	private:

		Real				m_t;
		Real				m_k;
		Real				m_restoration;
		Real				m_c;
		Real				m_r;
		Real				m_m;
		Real				m_h;
		Real				m_wind;
		Real				m_turbulence;

		SpatialVector		m_center;
		Real				m_radius;

		Matrix<4,4,Real>	m_invA;
		Matrix<4,4,Real>	m_dfdx;
		Matrix<4,4,Real>	m_hdfdx;
};

} /* namespace */

#endif /* __grass_Blade__ */

