/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#include "grass/grass.pmh"

using namespace fe;

int main(void)
{
	UNIT_START();
	BWORD completed=FALSE;

	try
	{
		Blade blade;
		std::vector<SpatialVector> tangent;		// root directions
		std::vector<SpatialVector> location;	// point locations
		std::vector<Real> velocity;				// angular velocities

		tangent.resize(1);
		set(location[0],1.0,0.0,0.0);

		location.resize(5);
		for(U32 m=0;m<5;m++)
		{
			set(location[m],0.0,m,0.0);
		}

		velocity.resize(4);
		for(U32 m=0;m<4;m++)
		{
			velocity[m]=0.0;
		}

		for(U32 n=0;n<1e6;n++)
		{
			feLog("\n");

			blade.step(0,tangent,location,velocity);
			for(U32 m=0;m<5;m++)
			{
				feLog("location %s  velocity %.6G\n",
						c_print(location[m]),m? velocity[m-1]: 0.0);
			}

			if(true)
			{
				for(I32 m=3;m>=0;m--)
				{
					for(I32 n=3;n>=0;n--)
					{
						Real along=n/4.0;
						Real between=along*location[m+1][0]+
								(1.0-along)*location[m][0];
						feLog("%*s#\n",int((between+4.0)*10)," ");
					}
				}
			}
		}

		completed=TRUE;
	}
	catch(Exception &e)	{ e.log(); }
	catch(...)			{ feLog("uncaught exception\n"); }

	UNIT_TEST(completed);
	UNIT_TRACK(1);
	UNIT_RETURN();
}
